local COLORS = {
	Color(1, 1, 0),
	Color(1, 0, 0),
	Color(1, 0, 1)
}
local MULTIPLIERS = {
	5,
	10
}

function CopBase:_update_aura(buff_name)
	if buff_name ~= "base_damage" then
		return
	end

	local buff = self:get_total_buff("base_damage")
	if buff > 0 then
		if not self._destroy_aura_key then
			self._destroy_aura_key = "destroy_aura" .. tostring(self._unit:key())
			self._unit:character_damage():add_listener(self._destroy_aura_key, { "death" }, callback(self, self, "_destroy_aura"))
		end

		if not alive(self._aura_light) then
			self._aura_light = World:create_light("spot|specular")
			self._aura_light:set_spot_angle_end(13)
			self._aura_light:set_near_range(400)
			self._aura_light:set_far_range(700)
			self._aura_light:link(self._unit:orientation_object())
			self._aura_light:set_local_rotation(Rotation:look_at(math.X, Vector3(0, 200, 600)))
			self._aura_light:set_local_position(Vector3(5, 200, 600))
		end

		local progress = math.map_range_clamped(buff, 1, 5, 0, 1)

		local color_index_frac = math.lerp(1, #COLORS, progress)
		local color_index_start = math.floor(color_index_frac)
		local color_index_end = math.min(color_index_start + 1, #COLORS)
		local color_progress = color_index_frac - color_index_start
		local color = math.lerp(COLORS[color_index_start], COLORS[color_index_end], color_progress)

		self._aura_light:set_color(color)

		local multiplier_index_frac = math.lerp(1, #MULTIPLIERS, progress)
		local multiplier_index_start = math.floor(multiplier_index_frac)
		local multiplier_index_end = math.min(multiplier_index_start + 1, #MULTIPLIERS)
		local multiplier_progress = multiplier_index_frac - multiplier_index_start
		local multiplier = math.lerp(MULTIPLIERS[multiplier_index_start], MULTIPLIERS[multiplier_index_end], multiplier_progress)

		self._aura_light:set_multiplier(multiplier)
	else
		self:_destroy_aura()
	end
end

function CopBase:_destroy_aura()
	if alive(self._aura_light) then
		World:delete_light(self._aura_light)
		self._aura_light = nil
	end
end

Hooks:PostHook(CopBase, "_refresh_buff_total", "_refresh_buff_total_edbi", CopBase._update_aura)
Hooks:PostHook(CopBase, "_sync_buff_total", "_sync_buff_total_edbi", CopBase._update_aura)
Hooks:PreHook(CopBase, "pre_destroy", "pre_destroy_edbi", CopBase._destroy_aura)
